@@ -1,4 +1,5 @@
 import * as yaml from "https://deno.land/std@0.170.0/encoding/yaml.ts";
+import $ from "https://deno.land/x/dax@0.33.0/mod.ts";
 
 enum OperatingSystem {
   Mac = "macOS-latest",
@@ -10,6 +11,8 @@ interface ProfileData {
   os: OperatingSystem;
   target: string;
   runTests?: boolean;
+  /** Build using cross. */
+  cross?: boolean;
 }
 
 const profileDataItems: ProfileData[] = [{
@@ -33,6 +36,10 @@ const profileDataItems: ProfileData[] = [{
 }, {
   os: OperatingSystem.Linux,
   target: "aarch64-unknown-linux-gnu",
+}, {
+  os: OperatingSystem.Linux,
+  target: "aarch64-unknown-linux-musl",
+  cross: true,
 }];
 const profiles = profileDataItems.map(profile => {
   return {
@@ -76,6 +83,7 @@ const ci = {
             os: profile.os,
             run_tests: (profile.runTests ?? false).toString(),
             target: profile.target,
+            cross: (profile.cross ?? false).toString(),
           })),
         },
       },
@@ -86,7 +94,7 @@ const ci = {
       },
       outputs: Object.fromEntries(
         profiles.map(profile => {
-          const entries = [];
+          const entries: string[][] = [];
           entries.push([
             profile.zipChecksumEnvVarName,
             "${{steps.pre_release_" + profile.target.replaceAll("-", "_") + ".outputs.ZIP_CHECKSUM}}",
@@ -104,10 +112,7 @@ const ci = {
         { name: "Checkout", uses: "actions/checkout@v2" },
         { uses: "dsherret/rust-toolchain-file@v1" },
         { uses: "Swatinem/rust-cache@v2" },
-        {
-          uses: "denoland/setup-deno@v1",
-          if: "matrix.config.target == 'x86_64-unknown-linux-gnu'",
-        },
+        { uses: "denoland/setup-deno@v1" },
         {
           name: "Verify wasmer-compiler version",
           if: "matrix.config.target == 'x86_64-unknown-linux-gnu'",
@@ -137,10 +142,17 @@ const ci = {
           if: "matrix.config.target == 'aarch64-unknown-linux-gnu'",
           run: [
             "sudo apt update",
-            "sudo apt install -y gcc-aarch64-linux-gnu",
+            "sudo apt install gcc-aarch64-linux-gnu",
             "rustup target add aarch64-unknown-linux-gnu",
           ].join("\n"),
         },
+        {
+          name: "Setup cross",
+          if: "matrix.config.cross == 'true'",
+          run: [
+            "cargo install cross --git https://github.com/cross-rs/cross --rev 44011c8854cb2eaac83b173cc323220ccdff18ea",
+          ].join("\n"),
+        },
         {
           name: "Setup (Mac aarch64)",
           if: "matrix.config.target == 'aarch64-apple-darwin'",
@@ -148,7 +160,7 @@ const ci = {
         },
         {
           name: "Build (Debug)",
-          if: "!startsWith(github.ref, 'refs/tags/')",
+          if: "matrix.config.cross != 'true' && !startsWith(github.ref, 'refs/tags/')",
           env: {
             "CARGO_TARGET_AARCH64_UNKNOWN_LINUX_GNU_LINKER": "aarch64-linux-gnu-gcc",
           },
@@ -158,7 +170,7 @@ const ci = {
         },
         {
           name: "Build (Release)",
-          if: "startsWith(github.ref, 'refs/tags/')",
+          if: "matrix.config.cross != 'true' && startsWith(github.ref, 'refs/tags/')",
           env: {
             "CARGO_TARGET_AARCH64_UNKNOWN_LINUX_GNU_LINKER": "aarch64-linux-gnu-gcc",
           },
@@ -166,6 +178,20 @@ const ci = {
             "cargo build -p dprint --locked --target ${{matrix.config.target}} --release",
           ].join("\n"),
         },
+        {
+          name: "Build cross (Debug)",
+          if: "matrix.config.cross == 'true' && !startsWith(github.ref, 'refs/tags/')",
+          run: [
+            "cross build -p dprint --locked --target ${{matrix.config.target}}",
+          ].join("\n"),
+        },
+        {
+          name: "Build cross (Release)",
+          if: "matrix.config.cross == 'true' && startsWith(github.ref, 'refs/tags/')",
+          run: [
+            "cross build -p dprint --locked --target ${{matrix.config.target}} --release",
+          ].join("\n"),
+        },
         {
           name: "Test (Debug)",
           if: "matrix.config.run_tests == 'true' && !startsWith(github.ref, 'refs/tags/')",
@@ -245,6 +271,7 @@ const ci = {
         }),
         {
           name: "Test shell installer",
+          if: "matrix.config.run_tests == 'true' && !startsWith(github.ref, 'refs/tags/')",
           run: [
             "cd website/src/assets",
             "chmod +x install.sh",
@@ -253,20 +280,19 @@ const ci = {
         },
         {
           name: "Test powershell installer (Windows)",
-          if: "startsWith(matrix.config.os, 'windows')",
+          if: "matrix.config.run_tests == 'true' && !startsWith(github.ref, 'refs/tags/') && startsWith(matrix.config.os, 'windows')",
           shell: "pwsh",
           run: ["cd website/src/assets", "./install.ps1"].join("\n"),
         },
-        {
-          name: "Test npm",
-          run: [
-            "cd deployment/npm",
-            "curl --fail --location --progress-bar --output \"SHASUMS256.txt\" \"https://github.com/dprint/dprint/releases/download/0.34.1/SHASUMS256.txt\"",
-            "node setup.js 0.34.1",
-            "npm install",
-            "node bin.js -v",
-          ].join("\n"),
-        },
+        // todo: temporarily ignore for aarch64-musl because a release hasn't been done with this
+        // {
+        //   name: "Test npm",
+        //   if: "matrix.config.run_tests == 'true' && !startsWith(github.ref, 'refs/tags/')",
+        //   run: [
+        //     "cd deployment/npm",
+        //     "deno run -A build.ts 0.37.1",
+        //   ].join("\n"),
+        // },
       ],
     },
     draft_release: {
@@ -367,3 +393,5 @@ finalText += yaml.stringify(ci, {
 });
 
 Deno.writeTextFileSync(new URL("./ci.yml", import.meta.url), finalText);
+
+await $`dprint fmt --log-level=warn "**/*.yml"`;
