@@ -2,23 +2,26 @@ use anyhow::anyhow;
 use anyhow::bail;
 use anyhow::Context as AnyhowContext;
 use anyhow::Result;
-use parking_lot::Mutex;
 use serde::de::DeserializeOwned;
+use std::cell::RefCell;
 use std::io::BufRead;
+use std::io::ErrorKind;
 use std::io::Read;
 use std::io::Write;
 use std::path::Path;
 use std::path::PathBuf;
 use std::process::Child;
 use std::process::ChildStderr;
-use std::process::ChildStdout;
 use std::process::Command;
 use std::process::Stdio;
+use std::rc::Rc;
 use std::sync::Arc;
 use std::time::Duration;
 use tokio::sync::oneshot;
 use tokio_util::sync::CancellationToken;
 
+use super::messages::CheckConfigUpdatesMessageBody;
+use super::messages::CheckConfigUpdatesResponseBody;
 use super::messages::FormatMessageBody;
 use super::messages::HostFormatMessageBody;
 use super::messages::MessageBody;
@@ -26,47 +29,60 @@ use super::messages::ProcessPluginMessage;
 use super::messages::RegisterConfigMessageBody;
 use super::messages::ResponseBody;
 use super::PLUGIN_SCHEMA_VERSION;
-use crate::communication::ArcFlag;
-use crate::communication::ArcIdStore;
+use crate::async_runtime::DropGuardAction;
+use crate::async_runtime::LocalBoxFuture;
+use crate::communication::AtomicFlag;
 use crate::communication::IdGenerator;
 use crate::communication::MessageReader;
 use crate::communication::MessageWriter;
-use crate::communication::Poisoner;
+use crate::communication::RcIdStore;
 use crate::communication::SingleThreadMessageWriter;
 use crate::configuration::ConfigKeyMap;
 use crate::configuration::ConfigurationDiagnostic;
 use crate::configuration::GlobalConfiguration;
+use crate::plugins::ConfigChange;
 use crate::plugins::CriticalFormatError;
+use crate::plugins::FileMatchingInfo;
+use crate::plugins::FormatConfigId;
 use crate::plugins::FormatRange;
 use crate::plugins::FormatResult;
-use crate::plugins::Host;
 use crate::plugins::HostFormatRequest;
 use crate::plugins::NullCancellationToken;
 use crate::plugins::PluginInfo;
 
 type DprintCancellationToken = Arc<dyn super::super::CancellationToken>;
 
+pub type HostFormatCallback = Rc<dyn Fn(HostFormatRequest) -> LocalBoxFuture<'static, FormatResult>>;
+
+pub struct ProcessPluginCommunicatorFormatRequest {
+  pub file_path: PathBuf,
+  pub file_text: String,
+  pub range: FormatRange,
+  pub config_id: FormatConfigId,
+  pub override_config: ConfigKeyMap,
+  pub on_host_format: HostFormatCallback,
+  pub token: DprintCancellationToken,
+}
+
 enum MessageResponseChannel {
   Acknowledgement(oneshot::Sender<Result<()>>),
   Data(oneshot::Sender<Result<Vec<u8>>>),
   Format(oneshot::Sender<Result<Option<Vec<u8>>>>),
 }
 
-#[derive(Clone)]
 struct Context {
   stdin_writer: SingleThreadMessageWriter<ProcessPluginMessage>,
-  poisoner: Poisoner,
-  shutdown_flag: ArcFlag,
+  shutdown_flag: Arc<AtomicFlag>,
   id_generator: IdGenerator,
-  messages: ArcIdStore<MessageResponseChannel>,
-  format_request_tokens: ArcIdStore<Arc<CancellationToken>>,
-  host: Arc<dyn Host>,
+  messages: RcIdStore<MessageResponseChannel>,
+  format_request_tokens: RcIdStore<Arc<CancellationToken>>,
+  host_format_callbacks: RcIdStore<HostFormatCallback>,
 }
 
 /// Communicates with a process plugin.
 pub struct ProcessPluginCommunicator {
-  child: Mutex<Option<Child>>,
-  context: Context,
+  child: RefCell<Option<Child>>,
+  context: Rc<Context>,
 }
 
 impl Drop for ProcessPluginCommunicator {
@@ -76,28 +92,22 @@ impl Drop for ProcessPluginCommunicator {
 }
 
 impl ProcessPluginCommunicator {
-  pub async fn new(executable_file_path: &Path, on_std_err: impl Fn(String) + Clone + Send + Sync + 'static, host: Arc<dyn Host>) -> Result<Self> {
-    ProcessPluginCommunicator::new_internal(executable_file_path, false, on_std_err, host).await
+  pub async fn new(executable_file_path: &Path, on_std_err: impl Fn(String) + Clone + Send + Sync + 'static) -> Result<Self> {
+    ProcessPluginCommunicator::new_internal(executable_file_path, false, on_std_err).await
   }
 
   /// Provides the `--init` CLI flag to tell the process plugin to do any initialization necessary
-  pub async fn new_with_init(executable_file_path: &Path, on_std_err: impl Fn(String) + Clone + Send + Sync + 'static, host: Arc<dyn Host>) -> Result<Self> {
-    ProcessPluginCommunicator::new_internal(executable_file_path, true, on_std_err, host).await
+  pub async fn new_with_init(executable_file_path: &Path, on_std_err: impl Fn(String) + Clone + Send + Sync + 'static) -> Result<Self> {
+    ProcessPluginCommunicator::new_internal(executable_file_path, true, on_std_err).await
   }
 
-  async fn new_internal(
-    executable_file_path: &Path,
-    is_init: bool,
-    on_std_err: impl Fn(String) + Clone + Send + Sync + 'static,
-    host: Arc<dyn Host>,
-  ) -> Result<Self> {
+  async fn new_internal(executable_file_path: &Path, is_init: bool, on_std_err: impl Fn(String) + Clone + Send + Sync + 'static) -> Result<Self> {
     let mut args = vec!["--parent-pid".to_string(), std::process::id().to_string()];
     if is_init {
       args.push("--init".to_string());
     }
 
-    let poisoner = Poisoner::default();
-    let shutdown_flag = ArcFlag::default();
+    let shutdown_flag = Arc::new(AtomicFlag::default());
     let mut child = Command::new(executable_file_path)
       .args(&args)
       .stdin(Stdio::piped())
@@ -108,12 +118,11 @@ impl ProcessPluginCommunicator {
 
     // read and output stderr prefixed
     let stderr = child.stderr.take().unwrap();
-    tokio::task::spawn_blocking({
-      let poisoner = poisoner.clone();
+    crate::async_runtime::spawn_blocking({
       let shutdown_flag = shutdown_flag.clone();
       let on_std_err = on_std_err.clone();
       move || {
-        std_err_redirect(poisoner, shutdown_flag, stderr, on_std_err);
+        std_err_redirect(shutdown_flag, stderr, on_std_err);
       }
     });
 
@@ -121,69 +130,115 @@ impl ProcessPluginCommunicator {
     let mut stdout_reader = MessageReader::new(child.stdout.take().unwrap());
     let mut stdin_writer = MessageWriter::new(child.stdin.take().unwrap());
 
-    verify_plugin_schema_version(&mut stdout_reader, &mut stdin_writer)
-      .context("Failed plugin schema verification. This may indicate you are using an old version of the dprint CLI or plugin and should upgrade.")?;
+    let (mut stdout_reader, stdin_writer, schema_version) = tokio::task::spawn_blocking(move || {
+      let schema_version = get_plugin_schema_version(&mut stdout_reader, &mut stdin_writer)
+        .context("Failed plugin schema verification. This may indicate you are using an old version of the dprint CLI or plugin and should upgrade")?;
+      Ok::<_, anyhow::Error>((stdout_reader, stdin_writer, schema_version))
+    })
+    .await??;
 
-    let stdin_writer = SingleThreadMessageWriter::for_stdin(stdin_writer, poisoner.clone());
-    let context = Context {
+    if schema_version != PLUGIN_SCHEMA_VERSION {
+      // kill the child to prevent it from ouputting to stderr
+      let _ = child.kill();
+      if schema_version < PLUGIN_SCHEMA_VERSION {
+        bail!(
+          "This plugin is too old to run in the dprint CLI and you will need to manually upgrade it (version was {}, but expected {}).\n\nUpgrade instructions: https://github.com/dprint/dprint/issues/731",
+          schema_version,
+          PLUGIN_SCHEMA_VERSION
+        );
+      } else {
+        bail!(
+          "Your dprint CLI is too old to run this plugin (version was {}, but expected {}). Try running: dprint upgrade",
+          schema_version,
+          PLUGIN_SCHEMA_VERSION
+        );
+      }
+    }
+
+    let stdin_writer = SingleThreadMessageWriter::for_stdin(stdin_writer);
+    let context = Rc::new(Context {
       id_generator: Default::default(),
       shutdown_flag,
       stdin_writer,
-      poisoner,
       messages: Default::default(),
       format_request_tokens: Default::default(),
-      host,
-    };
+      host_format_callbacks: Default::default(),
+    });
 
     // read from stdout
-    tokio::task::spawn_blocking({
-      let context = context.clone();
+    let (tx, mut rx) = tokio::sync::mpsc::unbounded_channel();
+    crate::async_runtime::spawn_blocking({
+      let shutdown_flag = context.shutdown_flag.clone();
+      let on_std_err = on_std_err.clone();
       move || {
         loop {
-          if let Err(err) = read_stdout_message(&mut stdout_reader, &context) {
-            if !context.poisoner.is_poisoned() && !context.shutdown_flag.is_raised() {
+          match ProcessPluginMessage::read(&mut stdout_reader) {
+            Ok(message) => {
+              if tx.send(message).is_err() {
+                break; // closed
+              }
+            }
+            Err(err) if err.kind() == ErrorKind::BrokenPipe => {
+              break;
+            }
+            Err(err) => {
+              if !shutdown_flag.is_raised() {
+                on_std_err(format!("Error reading stdout message: {:#}", err));
+              }
+              break;
+            }
+          }
+        }
+      }
+    });
+    crate::async_runtime::spawn({
+      let context = context.clone();
+      async move {
+        while let Some(message) = rx.recv().await {
+          if let Err(err) = handle_stdout_message(message, &context) {
+            if !context.shutdown_flag.is_raised() {
               on_std_err(format!("Error reading stdout message: {:#}", err));
             }
             break;
           }
         }
-        context.poisoner.poison();
+        // clear out all the messages
+        context.messages.take_all();
       }
     });
 
     Ok(Self {
-      child: Mutex::new(Some(child)),
+      child: RefCell::new(Some(child)),
       context,
     })
   }
 
   /// Perform a graceful shutdown.
   pub async fn shutdown(&self) {
-    self.context.shutdown_flag.raise();
-    if self.context.poisoner.is_poisoned() {
-      self.kill();
-    } else {
+    if self.context.shutdown_flag.raise() {
       // attempt to exit nicely
       tokio::select! {
         // we wait for acknowledgement in order to give the process
         // plugin a chance to clean up (ex. in case it has spawned
         // any processes it needs to kill or something like that)
         _ = self.send_with_acknowledgement(MessageBody::Close) => {}
-        _ = tokio::time::sleep(Duration::from_millis(250)) => {}
+        _ = tokio::time::sleep(Duration::from_millis(250)) => {
+          self.kill();
+        }
       }
-      self.context.poisoner.poison();
+    } else {
+      self.kill();
     }
   }
 
   pub fn kill(&self) {
     self.context.shutdown_flag.raise();
-    self.context.poisoner.poison();
-    if let Some(mut child) = self.child.lock().take() {
+    if let Some(mut child) = self.child.borrow_mut().take() {
       let _ignore = child.kill();
     }
   }
 
-  pub async fn register_config(&self, config_id: u32, global_config: &GlobalConfiguration, plugin_config: &ConfigKeyMap) -> Result<()> {
+  pub async fn register_config(&self, config_id: FormatConfigId, global_config: &GlobalConfiguration, plugin_config: &ConfigKeyMap) -> Result<()> {
     let global_config = serde_json::to_vec(global_config)?;
     let plugin_config = serde_json::to_vec(plugin_config)?;
     self
@@ -196,7 +251,7 @@ impl ProcessPluginCommunicator {
     Ok(())
   }
 
-  pub async fn release_config(&self, config_id: u32) -> Result<()> {
+  pub async fn release_config(&self, config_id: FormatConfigId) -> Result<()> {
     self.send_with_acknowledgement(MessageBody::ReleaseConfig(config_id)).await?;
     Ok(())
   }
@@ -213,41 +268,49 @@ impl ProcessPluginCommunicator {
     self.send_receiving_string(MessageBody::GetLicenseText).await
   }
 
-  pub async fn resolved_config(&self, config_id: u32) -> Result<String> {
+  pub async fn resolved_config(&self, config_id: FormatConfigId) -> Result<String> {
     self.send_receiving_string(MessageBody::GetResolvedConfig(config_id)).await
   }
 
-  pub async fn config_diagnostics(&self, config_id: u32) -> Result<Vec<ConfigurationDiagnostic>> {
+  pub async fn file_matching_info(&self, config_id: FormatConfigId) -> Result<FileMatchingInfo> {
+    self.send_receiving_data(MessageBody::GetFileMatchingInfo(config_id)).await
+  }
+
+  pub async fn config_diagnostics(&self, config_id: FormatConfigId) -> Result<Vec<ConfigurationDiagnostic>> {
     self.send_receiving_data(MessageBody::GetConfigDiagnostics(config_id)).await
   }
 
-  pub async fn format_text(
-    &self,
-    file_path: PathBuf,
-    file_text: String,
-    range: FormatRange,
-    config_id: u32,
-    override_config: ConfigKeyMap,
-    token: DprintCancellationToken,
-  ) -> FormatResult {
+  pub async fn check_config_updates(&self, plugin_config: ConfigKeyMap) -> Result<Vec<ConfigChange>> {
+    let message = CheckConfigUpdatesMessageBody { config: plugin_config };
+    let bytes = serde_json::to_vec(&message)?;
+    let response: CheckConfigUpdatesResponseBody = self.send_receiving_data(MessageBody::CheckConfigUpdates(bytes)).await?;
+    Ok(response.changes)
+  }
+
+  pub async fn format_text(&self, request: ProcessPluginCommunicatorFormatRequest) -> FormatResult {
     let (tx, rx) = oneshot::channel::<Result<Option<Vec<u8>>>>();
 
+    let message_id = self.context.id_generator.next();
+    let store_guard = self.context.host_format_callbacks.store_with_guard(message_id, request.on_host_format);
     let maybe_result = self
-      .send_message(
+      .send_message_with_id(
+        message_id,
         MessageBody::Format(FormatMessageBody {
-          file_path,
-          file_text: file_text.into_bytes(),
-          range,
-          config_id,
-          override_config: serde_json::to_vec(&override_config).unwrap(),
+          file_path: request.file_path,
+          file_text: request.file_text.into_bytes(),
+          range: request.range,
+          config_id: request.config_id,
+          override_config: serde_json::to_vec(&request.override_config).unwrap(),
         }),
         MessageResponseChannel::Format(tx),
         rx,
-        token.clone(),
+        request.token.clone(),
       )
       .await;
 
-    if token.is_cancelled() {
+    drop(store_guard); // explicit for clarity
+
+    if request.token.is_cancelled() {
       Ok(None)
     } else {
       match maybe_result {
@@ -261,7 +324,11 @@ impl ProcessPluginCommunicator {
 
   /// Checks if the process is functioning.
   pub async fn is_process_alive(&self) -> bool {
-    !self.context.poisoner.is_poisoned() || self.ask_is_alive().await
+    if self.context.shutdown_flag.is_raised() {
+      false
+    } else {
+      self.ask_is_alive().await
+    }
   }
 
   async fn send_with_acknowledgement(&self, body: MessageBody) -> Result<()> {
@@ -296,22 +363,39 @@ impl ProcessPluginCommunicator {
     token: Arc<dyn super::super::CancellationToken>,
   ) -> Result<Result<T>> {
     let message_id = self.context.id_generator.next();
+    self.send_message_with_id(message_id, body, response_channel, receiver, token).await
+  }
+
+  async fn send_message_with_id<T: Default>(
+    &self,
+    message_id: u32,
+    body: MessageBody,
+    response_channel: MessageResponseChannel,
+    receiver: oneshot::Receiver<Result<T>>,
+    token: Arc<dyn super::super::CancellationToken>,
+  ) -> Result<Result<T>> {
+    let mut drop_guard = DropGuardAction::new(|| {
+      // clear up memory
+      self.context.messages.take(message_id);
+      // send cancellation to the client
+      let _ = self.context.stdin_writer.send(ProcessPluginMessage {
+        id: self.context.id_generator.next(),
+        body: MessageBody::CancelFormat(message_id),
+      });
+    });
+
     self.context.messages.store(message_id, response_channel);
     self.context.stdin_writer.send(ProcessPluginMessage { id: message_id, body })?;
     tokio::select! {
-      _ = self.context.poisoner.wait_poisoned() => {
-        self.context.messages.take(message_id); // clear memory
-        bail!("Sending message failed because the process plugin failed.");
-      }
       _ = token.wait_cancellation() => {
-        self.context.messages.take(message_id); // clear memory
+        drop(drop_guard); // explicit
         Ok(Ok(Default::default()))
       }
       response = receiver => {
+        drop_guard.forget(); // we completed, so don't run the drop guard
         match response {
           Ok(data) => Ok(data),
           Err(err) => {
-            self.context.poisoner.poison();
             bail!("Error waiting on message ({}). {:#}", message_id, err)
           }
         }
@@ -320,10 +404,7 @@ impl ProcessPluginCommunicator {
   }
 }
 
-fn verify_plugin_schema_version<TRead: Read + Unpin, TWrite: Write + Unpin>(
-  reader: &mut MessageReader<TRead>,
-  writer: &mut MessageWriter<TWrite>,
-) -> Result<()> {
+fn get_plugin_schema_version<TRead: Read + Unpin, TWrite: Write + Unpin>(reader: &mut MessageReader<TRead>, writer: &mut MessageWriter<TWrite>) -> Result<u32> {
   // since this is the setup, use a lot of contexts to find exactly where it failed
   writer.send_u32(0).context("Failed asking for schema version.")?; // ask for schema version
   writer.flush().context("Failed flushing schema version request.")?;
@@ -331,30 +412,16 @@ fn verify_plugin_schema_version<TRead: Read + Unpin, TWrite: Write + Unpin>(
   if acknowledgement_response != 0 {
     bail!("Plugin response was unexpected ({acknowledgement_response}).");
   }
-  let plugin_schema_version = reader.read_u32().context("Could not read schema version.")?;
-  if plugin_schema_version != PLUGIN_SCHEMA_VERSION {
-    bail!(
-      "The plugin schema version was {}, but expected {}.",
-      plugin_schema_version,
-      PLUGIN_SCHEMA_VERSION
-    );
-  }
-
-  Ok(())
+  reader.read_u32().context("Could not read schema version.")
 }
 
-fn std_err_redirect(poisoner: Poisoner, shutdown_flag: ArcFlag, stderr: ChildStderr, on_std_err: impl Fn(String) + Send + Sync + 'static) {
-  use std::io::ErrorKind;
+fn std_err_redirect(shutdown_flag: Arc<AtomicFlag>, stderr: ChildStderr, on_std_err: impl Fn(String) + Send + Sync + 'static) {
   let reader = std::io::BufReader::new(stderr);
   for line in reader.lines() {
     match line {
       Ok(line) => on_std_err(line),
       Err(err) => {
-        if poisoner.is_poisoned() || shutdown_flag.is_raised() {
-          return;
-        }
-        if err.kind() == ErrorKind::BrokenPipe {
-          poisoner.poison();
+        if shutdown_flag.is_raised() || err.kind() == ErrorKind::BrokenPipe {
           return;
         } else {
           on_std_err(format!("Error reading line from process plugin stderr. {:#}", err));
@@ -364,9 +431,7 @@ fn std_err_redirect(poisoner: Poisoner, shutdown_flag: ArcFlag, stderr: ChildStd
   }
 }
 
-fn read_stdout_message(reader: &mut MessageReader<ChildStdout>, context: &Context) -> Result<()> {
-  let message = ProcessPluginMessage::read(reader)?;
-
+fn handle_stdout_message(message: ProcessPluginMessage, context: &Rc<Context>) -> Result<()> {
   match message.body {
     MessageBody::Success(message_id) => match context.messages.take(message_id) {
       Some(MessageResponseChannel::Acknowledgement(channel)) => {
@@ -423,12 +488,14 @@ fn read_stdout_message(reader: &mut MessageReader<ChildStdout>, context: &Contex
       if let Some(token) = context.format_request_tokens.take(message_id) {
         token.cancel();
       }
+      context.host_format_callbacks.take(message_id);
+      // do not clear from context.messages here because the cancellation will do that
     }
     MessageBody::HostFormat(body) => {
       // spawn a task to do the host formatting, then send a message back to the
       // plugin with the result
       let context = context.clone();
-      tokio::task::spawn(async move {
+      crate::async_runtime::spawn(async move {
         let result = host_format(context.clone(), message.id, body).await;
 
         // ignore failure, as this means that the process shut down
@@ -462,7 +529,9 @@ fn read_stdout_message(reader: &mut MessageReader<ChildStdout>, context: &Contex
     | MessageBody::RegisterConfig(_)
     | MessageBody::ReleaseConfig(_)
     | MessageBody::GetConfigDiagnostics(_)
-    | MessageBody::GetResolvedConfig(_) => {
+    | MessageBody::GetFileMatchingInfo(_)
+    | MessageBody::GetResolvedConfig(_)
+    | MessageBody::CheckConfigUpdates(_) => {
       let _ = context.stdin_writer.send(ProcessPluginMessage {
         id: context.id_generator.next(),
         body: MessageBody::Error(ResponseBody {
@@ -481,23 +550,23 @@ fn read_stdout_message(reader: &mut MessageReader<ChildStdout>, context: &Contex
   Ok(())
 }
 
-async fn host_format(context: Context, message_id: u32, body: HostFormatMessageBody) -> FormatResult {
+async fn host_format(context: Rc<Context>, message_id: u32, body: HostFormatMessageBody) -> FormatResult {
+  let file_text = String::from_utf8(body.file_text)?; // surface error before storing token
+
+  let Some(callback) = context.host_format_callbacks.get_cloned(body.original_message_id) else {
+    return FormatResult::Err(anyhow!("Could not find host format callback for message id: {}", body.original_message_id));
+  };
+
   let token = Arc::new(CancellationToken::new());
-  context.format_request_tokens.store(message_id, token.clone());
-
-  let result = context
-    .host
-    .format(HostFormatRequest {
-      file_path: body.file_path,
-      file_text: String::from_utf8(body.file_text)?,
-      range: body.range,
-      override_config: serde_json::from_slice(&body.override_config).unwrap(),
-      token,
-    })
-    .await;
-
-  // don't leak cancellation tokens
-  context.format_request_tokens.take(message_id);
-
+  let store_guard = context.format_request_tokens.store_with_guard(message_id, token.clone());
+  let result = callback(HostFormatRequest {
+    file_path: body.file_path,
+    file_text,
+    range: body.range,
+    override_config: serde_json::from_slice(&body.override_config).unwrap(),
+    token,
+  })
+  .await;
+  drop(store_guard); // explicit for clarity
   result
 }
