@@ -1,26 +1,27 @@
 use anyhow::anyhow;
 use anyhow::bail;
-use anyhow::Error;
 use anyhow::Result;
-use futures::Future;
 use parking_lot::Condvar;
 use parking_lot::Mutex;
 use path_clean::PathClean;
 use std::collections::HashMap;
 use std::collections::HashSet;
+use std::future::Future;
 use std::io::Read;
 use std::io::Write;
 use std::path::Path;
 use std::path::PathBuf;
 use std::sync::Arc;
 
+use dprint_core::async_runtime::async_trait;
+
 use super::CanonicalizedPathBuf;
 use super::DirEntry;
-use super::DirEntryKind;
 use super::Environment;
 use super::FilePermissions;
 use super::UrlDownloader;
 use crate::plugins::CompilationResult;
+use crate::utils::LogLevel;
 
 #[derive(Default)]
 struct BufferData {
@@ -94,7 +95,7 @@ impl Write for TestPipeWriter {
 
 #[derive(Clone)]
 pub struct TestEnvironment {
-  is_verbose: Arc<Mutex<bool>>,
+  log_level: Arc<Mutex<LogLevel>>,
   cwd: Arc<Mutex<String>>,
   files: Arc<Mutex<HashMap<PathBuf, Vec<u8>>>>,
   file_permissions: Arc<Mutex<HashMap<PathBuf, FilePermissions>>>,
@@ -107,10 +108,9 @@ pub struct TestEnvironment {
   confirm_results: Arc<Mutex<Vec<Result<Option<bool>>>>>,
   is_stdout_machine_readable: Arc<Mutex<bool>>,
   wasm_compile_result: Arc<Mutex<Option<CompilationResult>>>,
-  dir_info_error: Arc<Mutex<Option<Error>>>,
+  dir_info_error: Arc<Mutex<Option<std::io::Error>>>,
   std_in_pipe: Arc<Mutex<(Option<TestPipeWriter>, TestPipeReader)>>,
   std_out_pipe: Arc<Mutex<(Option<TestPipeWriter>, TestPipeReader)>>,
-  runtime_handle: Arc<Mutex<Option<tokio::runtime::Handle>>>,
   #[cfg(windows)]
   path_dirs: Arc<Mutex<Vec<PathBuf>>>,
   cpu_arch: Arc<Mutex<String>>,
@@ -121,7 +121,7 @@ pub struct TestEnvironment {
 impl TestEnvironment {
   pub fn new() -> TestEnvironment {
     TestEnvironment {
-      is_verbose: Arc::new(Mutex::new(false)),
+      log_level: Arc::new(Mutex::new(LogLevel::Info)),
       cwd: Arc::new(Mutex::new(String::from("/"))),
       files: Default::default(),
       file_permissions: Default::default(),
@@ -143,7 +143,6 @@ impl TestEnvironment {
         let pipe = create_test_pipe();
         (Some(pipe.0), pipe.1)
       })),
-      runtime_handle: Default::default(),
       #[cfg(windows)]
       path_dirs: Default::default(),
       cpu_arch: Arc::new(Mutex::new("x86_64".to_string())),
@@ -170,33 +169,36 @@ impl TestEnvironment {
   }
 
   pub fn add_remote_file_bytes(&self, path: &str, bytes: Vec<u8>) {
-    let mut remote_files = self.remote_files.lock();
-    remote_files.insert(String::from(path), Ok(bytes));
+    self.remote_files.lock().insert(String::from(path), Ok(bytes));
   }
 
   pub fn add_remote_file_error(&self, path: &str, err: &str) {
-    let mut remote_files = self.remote_files.lock();
-    remote_files.insert(String::from(path), Err(anyhow!("{}", err)));
+    self.remote_files.lock().insert(String::from(path), Err(anyhow!("{}", err)));
+  }
+
+  pub fn get_remote_file(&self, url: &str) -> Result<Option<Vec<u8>>> {
+    let remote_files = self.remote_files.lock();
+    match remote_files.get(&String::from(url)) {
+      Some(Ok(result)) => Ok(Some(result.clone())),
+      Some(Err(err)) => Err(anyhow!("{:#}", err)),
+      None => Ok(None),
+    }
   }
 
   pub fn is_dir_deleted(&self, path: impl AsRef<Path>) -> bool {
-    let deleted_directories = self.deleted_directories.lock();
-    deleted_directories.contains(&path.as_ref().to_path_buf())
+    self.deleted_directories.lock().contains(&path.as_ref().to_path_buf())
   }
 
   pub fn set_selection_result(&self, index: usize) {
-    let mut selection_result = self.selection_result.lock();
-    *selection_result = index;
+    *self.selection_result.lock() = index;
   }
 
   pub fn set_multi_selection_result(&self, indexes: Vec<usize>) {
-    let mut multi_selection_result = self.multi_selection_result.lock();
-    *multi_selection_result = Some(indexes);
+    *self.multi_selection_result.lock() = Some(indexes);
   }
 
   pub fn set_confirm_results(&self, values: Vec<Result<Option<bool>>>) {
-    let mut confirm_results = self.confirm_results.lock();
-    *confirm_results = values;
+    *self.confirm_results.lock() = values;
   }
 
   pub fn set_cwd(&self, new_path: &str) {
@@ -208,14 +210,12 @@ impl TestEnvironment {
     *self.is_stdout_machine_readable.lock() = value;
   }
 
-  pub fn set_verbose(&self, value: bool) {
-    let mut is_verbose = self.is_verbose.lock();
-    *is_verbose = value;
+  pub fn set_log_level(&self, value: LogLevel) {
+    *self.log_level.lock() = value;
   }
 
   pub fn set_wasm_compile_result(&self, value: CompilationResult) {
-    let mut wasm_compile_result = self.wasm_compile_result.lock();
-    *wasm_compile_result = Some(value);
+    *self.wasm_compile_result.lock() = Some(value);
   }
 
   pub fn stdout_reader(&self) -> Box<dyn Read + Send> {
@@ -231,9 +231,8 @@ impl TestEnvironment {
     self.path_dirs.lock().clone()
   }
 
-  pub fn set_dir_info_error(&self, err: Error) {
-    let mut dir_info_error = self.dir_info_error.lock();
-    *dir_info_error = Some(err);
+  pub fn set_dir_info_error(&self, err: std::io::Error) {
+    *self.dir_info_error.lock() = Some(err);
   }
 
   pub fn set_current_exe_path(&self, path: impl AsRef<Path>) {
@@ -248,14 +247,9 @@ impl TestEnvironment {
     *self.max_threads_count.lock() = value;
   }
 
-  pub fn set_runtime_handle(&self, handle: tokio::runtime::Handle) {
-    *self.runtime_handle.lock() = Some(handle);
-  }
-
   /// Remember to drop the plugins collection manually if using this with one.
   pub fn run_in_runtime<T>(&self, future: impl Future<Output = T>) -> T {
-    let rt = tokio::runtime::Builder::new_multi_thread().enable_time().build().unwrap();
-    self.set_runtime_handle(rt.handle().clone());
+    let rt = tokio::runtime::Builder::new_current_thread().enable_time().build().unwrap();
     rt.block_on(future)
   }
 
@@ -291,17 +285,14 @@ impl Drop for TestEnvironment {
   }
 }
 
+#[async_trait(?Send)]
 impl UrlDownloader for TestEnvironment {
-  fn download_file(&self, url: &str) -> Result<Option<Vec<u8>>> {
-    let remote_files = self.remote_files.lock();
-    match remote_files.get(&String::from(url)) {
-      Some(Ok(result)) => Ok(Some(result.clone())),
-      Some(Err(err)) => Err(anyhow!("{:#}", err)),
-      None => Ok(None),
-    }
+  async fn download_file(&self, url: &str) -> Result<Option<Vec<u8>>> {
+    self.get_remote_file(url)
   }
 }
 
+#[async_trait]
 impl Environment for TestEnvironment {
   fn is_real(&self) -> bool {
     false
@@ -321,10 +312,6 @@ impl Environment for TestEnvironment {
     }
   }
 
-  fn write_file(&self, file_path: impl AsRef<Path>, file_text: &str) -> Result<()> {
-    self.write_file_bytes(file_path, file_text.as_bytes())
-  }
-
   fn write_file_bytes(&self, file_path: impl AsRef<Path>, bytes: &[u8]) -> Result<()> {
     let file_path = self.clean_path(file_path);
     let mut files = self.files.lock();
@@ -376,7 +363,7 @@ impl Environment for TestEnvironment {
     Ok(())
   }
 
-  fn dir_info(&self, dir_path: impl AsRef<Path>) -> Result<Vec<DirEntry>> {
+  fn dir_info(&self, dir_path: impl AsRef<Path>) -> std::io::Result<Vec<DirEntry>> {
     if let Some(err) = self.dir_info_error.lock().take() {
       return Err(err);
     }
@@ -388,8 +375,8 @@ impl Environment for TestEnvironment {
     let files = self.files.lock();
     for key in files.keys() {
       if key.parent().unwrap() == dir_path {
-        entries.push(DirEntry {
-          kind: DirEntryKind::File,
+        entries.push(DirEntry::File {
+          name: key.file_name().unwrap().to_os_string(),
           path: key.clone(),
         });
       } else {
@@ -401,10 +388,7 @@ impl Environment for TestEnvironment {
           };
 
           if ancestor_parent_dir == dir_path && found_directories.insert(ancestor_dir) {
-            entries.push(DirEntry {
-              kind: DirEntryKind::Directory,
-              path: ancestor_dir.to_path_buf(),
-            });
+            entries.push(DirEntry::Directory(ancestor_dir.to_path_buf()));
             break;
           }
           current_dir = ancestor_dir.parent();
@@ -459,7 +443,7 @@ impl Environment for TestEnvironment {
     Ok(self.current_exe_path.lock().clone())
   }
 
-  fn log(&self, text: &str) {
+  fn __log__(&self, text: &str) {
     if *self.is_stdout_machine_readable.lock() {
       return;
     }
@@ -481,7 +465,7 @@ impl Environment for TestEnvironment {
     action: TCreate,
     _: usize,
   ) -> TResult {
-    self.log_stderr(message);
+    self.__log_stderr__(message);
     action(Box::new(|_| {}))
   }
 
@@ -510,12 +494,12 @@ impl Environment for TestEnvironment {
   }
 
   fn get_selection(&self, prompt_message: &str, _: u16, _: &[String]) -> Result<usize> {
-    self.log_stderr(prompt_message);
+    self.__log_stderr__(prompt_message);
     Ok(*self.selection_result.lock())
   }
 
   fn get_multi_selection(&self, prompt_message: &str, _: u16, items: &[(bool, String)]) -> Result<Vec<usize>> {
-    self.log_stderr(prompt_message);
+    self.__log_stderr__(prompt_message);
     let default_values = items
       .iter()
       .enumerate()
@@ -527,7 +511,7 @@ impl Environment for TestEnvironment {
   fn confirm(&self, prompt_message: &str, default_value: bool) -> Result<bool> {
     let mut confirm_results = self.confirm_results.lock();
     let result = confirm_results.remove(0).map(|v| v.unwrap_or(default_value));
-    self.log_stderr(&format!(
+    self.__log_stderr__(&format!(
       "{} {}",
       prompt_message,
       match &result {
@@ -539,8 +523,12 @@ impl Environment for TestEnvironment {
     result
   }
 
-  fn is_verbose(&self) -> bool {
-    *self.is_verbose.lock()
+  fn is_ci(&self) -> bool {
+    false
+  }
+
+  fn log_level(&self) -> LogLevel {
+    *self.log_level.lock()
   }
 
   fn compile_wasm(&self, _: &[u8]) -> Result<CompilationResult> {
@@ -548,6 +536,14 @@ impl Environment for TestEnvironment {
     Ok(wasm_compile_result.clone().expect("Expected compilation result to be set."))
   }
 
+  fn wasm_cache_key(&self) -> String {
+    self.cpu_arch()
+  }
+
+  async fn cpu_usage(&self) -> u8 {
+    20
+  }
+
   fn stdout(&self) -> Box<dyn Write + Send> {
     Box::new(self.std_out_pipe.lock().0.take().unwrap())
   }
@@ -556,11 +552,6 @@ impl Environment for TestEnvironment {
     Box::new(self.std_in_pipe.lock().1.clone())
   }
 
-  fn runtime_handle(&self) -> tokio::runtime::Handle {
-    // need to call set_runtime_handle to make this not panic
-    self.runtime_handle.lock().as_ref().unwrap().clone()
-  }
-
   #[cfg(windows)]
   fn ensure_system_path(&self, directory_path: &str) -> Result<()> {
     let mut path_dirs = self.path_dirs.lock();
