@@ -1,7 +1,6 @@
 use anyhow::bail;
 use anyhow::Result;
 use dprint_core::plugins::process::ProcessPluginCommunicator;
-use dprint_core::plugins::NoopHost;
 use dprint_core::plugins::PluginInfo;
 use serde::Deserialize;
 use serde::Serialize;
@@ -9,7 +8,6 @@ use serde_json::Value;
 use std::path::Path;
 use std::path::PathBuf;
 use std::str;
-use std::sync::Arc;
 
 use crate::environment::Environment;
 use crate::plugins::implementations::SetupPluginResult;
@@ -48,7 +46,7 @@ pub async fn setup_process_plugin<TEnvironment: Environment>(
   plugin_file_bytes: &[u8],
   environment: &TEnvironment,
 ) -> Result<SetupPluginResult> {
-  let plugin_zip_bytes = get_plugin_zip_bytes(url_or_file_path, plugin_file_bytes, environment)?;
+  let plugin_zip_bytes = get_plugin_zip_bytes(url_or_file_path, plugin_file_bytes, environment).await?;
   let plugin_cache_dir_path = get_plugin_dir_path(&plugin_zip_bytes.name, &plugin_zip_bytes.version, environment);
 
   let result = setup_inner(&plugin_cache_dir_path, plugin_zip_bytes.name, &plugin_zip_bytes.zip_bytes, environment).await;
@@ -56,7 +54,7 @@ pub async fn setup_process_plugin<TEnvironment: Environment>(
   return match result {
     Ok(result) => Ok(result),
     Err(err) => {
-      log_verbose!(environment, "Failed setting up process plugin. {:#}", err);
+      log_debug!(environment, "Failed setting up process plugin. {:#}", err);
       // failed, so delete the dir if it exists
       let _ignore = environment.remove_dir_all(&plugin_cache_dir_path);
       Err(err)
@@ -69,13 +67,12 @@ pub async fn setup_process_plugin<TEnvironment: Environment>(
     zip_bytes: &[u8],
     environment: &TEnvironment,
   ) -> Result<SetupPluginResult> {
-    if environment.path_exists(plugin_cache_dir_path) {
-      environment.remove_dir_all(plugin_cache_dir_path)?;
-    }
+    let _ = environment.remove_dir_all(plugin_cache_dir_path);
+    environment.mk_dir_all(plugin_cache_dir_path)?;
+    let plugin_executable_file_path = get_plugin_executable_file_path(plugin_cache_dir_path, &plugin_name);
 
     extract_zip(&format!("Extracting zip for {}", plugin_name), zip_bytes, plugin_cache_dir_path, environment)?;
 
-    let plugin_executable_file_path = get_plugin_executable_file_path(plugin_cache_dir_path, &plugin_name);
     if !environment.path_exists(&plugin_executable_file_path) {
       bail!(
         "Plugin zip file did not contain required executable at: {}",
@@ -84,18 +81,15 @@ pub async fn setup_process_plugin<TEnvironment: Environment>(
     }
 
     let executable_path = super::get_test_safe_executable_path(plugin_executable_file_path.clone(), environment);
-    let communicator = ProcessPluginCommunicator::new_with_init(
-      &executable_path,
-      {
-        let environment = environment.clone();
-        move |error_message| {
+    let communicator = ProcessPluginCommunicator::new_with_init(&executable_path, {
+      let environment = environment.clone();
+      move |error_message| {
+        // consider messages from process plugins as warnings
+        if environment.log_level().is_warn() {
           environment.log_stderr_with_context(&error_message, &plugin_name);
         }
-      },
-      // it's ok to use a no-op host here because
-      // we're only getting the plugin information
-      Arc::new(NoopHost),
-    )
+      }
+    })
     .await?;
     let plugin_info = communicator.plugin_info().await?;
     communicator.shutdown().await;
@@ -125,6 +119,8 @@ struct ProcessPluginFile {
   linux_x86_64_musl: Option<ProcessPluginPath>,
   #[serde(rename = "linux-aarch64")]
   linux_aarch64: Option<ProcessPluginPath>,
+  #[serde(rename = "linux-aarch64-musl")]
+  linux_aarch64_musl: Option<ProcessPluginPath>,
   #[serde(rename = "darwin-x86_64")]
   darwin_x86_64: Option<ProcessPluginPath>,
   #[serde(rename = "darwin-aarch64")]
@@ -146,7 +142,7 @@ struct ProcessPluginZipBytes {
   zip_bytes: Vec<u8>,
 }
 
-fn get_plugin_zip_bytes<TEnvironment: Environment>(
+async fn get_plugin_zip_bytes<TEnvironment: Environment>(
   url_or_file_path: &PathSource,
   plugin_file_bytes: &[u8],
   environment: &TEnvironment,
@@ -154,7 +150,7 @@ fn get_plugin_zip_bytes<TEnvironment: Environment>(
   let plugin_file = deserialize_file(plugin_file_bytes)?;
   let plugin_path = get_os_path(&plugin_file, environment)?;
   let plugin_zip_path = resolve_url_or_file_path_to_path_source(&plugin_path.reference, &url_or_file_path.parent(), environment)?;
-  let plugin_zip_bytes = fetch_file_or_url_bytes(&plugin_zip_path, environment)?;
+  let plugin_zip_bytes = fetch_file_or_url_bytes(&plugin_zip_path, environment).await?;
   if let Err(err) = verify_sha256_checksum(&plugin_zip_bytes, &plugin_path.checksum) {
     bail!(
       concat!(
@@ -198,8 +194,10 @@ fn verify_plugin_file(plugin_file: &Value) -> Result<()> {
 
   let kind = plugin_file.as_object().and_then(|o| o.get("kind")).and_then(|v| v.as_str());
 
-  if kind.is_some() && kind != Some("process") {
-    bail!("Only process plugins are supported by this version of dprint. Please upgrade your CLI.");
+  if let Some(kind) = kind {
+    if kind != "process" {
+      bail!("Unsupported plugin kind: {kind}\nOnly process plugins are supported by this version of dprint. Please upgrade your CLI.");
+    }
   }
 
   Ok(())
@@ -216,6 +214,7 @@ fn get_os_path<'a>(plugin_file: &'a ProcessPluginFile, environment: &impl Enviro
     },
     "linux-musl" => match arch.as_str() {
       "x86_64" => plugin_file.linux_x86_64_musl.as_ref(),
+      "aarch64" => plugin_file.linux_aarch64_musl.as_ref().or(plugin_file.linux_x86_64_musl.as_ref()),
       _ => None,
     },
     "macos" => match arch.as_str() {
@@ -233,7 +232,7 @@ fn get_os_path<'a>(plugin_file: &'a ProcessPluginFile, environment: &impl Enviro
   match path {
     Some(path) => Ok(path),
     None => {
-      log_verbose!(environment, "Plugin File -- {:#?}", plugin_file);
+      log_debug!(environment, "Plugin File -- {:#?}", plugin_file);
       bail!("Unsupported CPU architecture: {} ({})", arch, os)
     }
   }
@@ -252,7 +251,7 @@ mod test {
         .err()
         .unwrap()
         .to_string(),
-      "Only process plugins are supported by this version of dprint. Please upgrade your CLI.",
+      "Unsupported plugin kind: other\nOnly process plugins are supported by this version of dprint. Please upgrade your CLI.",
     );
   }
 }
