@@ -1,41 +1,40 @@
 use anyhow::Result;
 use std::collections::HashMap;
 use std::path::Path;
-use std::sync::Arc;
+use std::rc::Rc;
 
 use crate::environment::CanonicalizedPathBuf;
 use crate::patterns::get_patterns_as_glob_matcher;
+use crate::resolution::PluginWithConfig;
 use crate::utils::get_lowercase_file_extension;
 use crate::utils::get_lowercase_file_name;
 use crate::utils::GlobMatcher;
 use crate::utils::GlobMatchesDetail;
 
-use super::Plugin;
-
 #[derive(Default)]
 pub struct PluginNameResolutionMaps {
   extension_to_plugin_names_map: HashMap<String, Vec<String>>,
   file_name_to_plugin_names_map: HashMap<String, Vec<String>>,
   /// Associations matchers ordered by precedence.
-  association_matchers: Vec<(String, Arc<GlobMatcher>)>,
+  association_matchers: Vec<(String, Rc<GlobMatcher>)>,
   /// Associations matchers in a map.
-  association_matchers_map: HashMap<String, Arc<GlobMatcher>>,
+  association_matchers_map: HashMap<String, Rc<GlobMatcher>>,
 }
 
 impl PluginNameResolutionMaps {
-  pub fn from_plugins(plugins: &[Box<dyn Plugin>], config_base_path: &CanonicalizedPathBuf) -> Result<Self> {
+  pub fn from_plugins<'a>(plugins: impl Iterator<Item = &'a PluginWithConfig>, config_base_path: &CanonicalizedPathBuf) -> Result<Self> {
     let mut plugin_name_maps = PluginNameResolutionMaps::default();
     for plugin in plugins {
       let plugin_name = plugin.name();
 
-      for extension in plugin.file_extensions() {
+      for extension in &plugin.file_matching.file_extensions {
         plugin_name_maps
           .extension_to_plugin_names_map
           .entry(extension.to_lowercase())
           .or_default()
           .push(plugin_name.to_string());
       }
-      for file_name in plugin.file_names() {
+      for file_name in &plugin.file_matching.file_names {
         plugin_name_maps
           .file_name_to_plugin_names_map
           .entry(file_name.to_lowercase())
@@ -43,8 +42,8 @@ impl PluginNameResolutionMaps {
           .push(plugin_name.to_string());
       }
 
-      if let Some(matcher) = get_plugin_association_glob_matcher(&**plugin, config_base_path)? {
-        let matcher = Arc::new(matcher);
+      if let Some(matcher) = get_plugin_association_glob_matcher(plugin, config_base_path)? {
+        let matcher = Rc::new(matcher);
         plugin_name_maps.association_matchers.push((plugin_name.to_string(), matcher.clone()));
         plugin_name_maps.association_matchers_map.insert(plugin_name.to_string(), matcher);
       }
@@ -97,8 +96,8 @@ impl PluginNameResolutionMaps {
   }
 }
 
-fn get_plugin_association_glob_matcher(plugin: &dyn Plugin, config_base_path: &CanonicalizedPathBuf) -> Result<Option<GlobMatcher>> {
-  Ok(if let Some(associations) = plugin.get_config().0.associations.as_ref() {
+fn get_plugin_association_glob_matcher(plugin: &PluginWithConfig, config_base_path: &CanonicalizedPathBuf) -> Result<Option<GlobMatcher>> {
+  Ok(if let Some(associations) = plugin.associations.as_ref() {
     Some(get_patterns_as_glob_matcher(associations, config_base_path)?)
   } else {
     None
