@@ -1,19 +1,32 @@
+use std::cell::RefCell;
 use std::io::Write;
 use std::path::PathBuf;
-use std::sync::Arc;
+use std::rc::Rc;
 
 use anyhow::Result;
 use crossterm::style::Stylize;
 use once_cell::sync::Lazy;
+use thiserror::Error;
 
 use crate::arg_parser::parse_args;
 use crate::environment::TestEnvironment;
 use crate::plugins::CompilationResult;
 use crate::plugins::PluginCache;
 use crate::plugins::PluginResolver;
-use crate::plugins::PluginsCollection;
 use crate::run_cli::run_cli;
 use crate::utils::TestStdInReader;
+use crate::AppError;
+
+// macro lifted from Deno's codebase
+#[macro_export]
+macro_rules! assert_contains {
+  ($string:expr, $($test:expr),+ $(,)?) => {
+    let string = &$string;
+    if !($(string.contains($test))||+) {
+      panic!("{:?} does not contain any of {:?}", string, [$($test),+]);
+    }
+  }
+}
 
 // this file should automatically be built when building the workspace
 pub static TEST_PROCESS_PLUGIN_PATH: Lazy<PathBuf> = Lazy::new(|| {
@@ -53,75 +66,162 @@ pub static PROCESS_PLUGIN_ZIP_BYTES: Lazy<Vec<u8>> = Lazy::new(|| {
   zip.write(&file_bytes).unwrap();
   zip.finish().unwrap().into_inner()
 });
+pub static PROCESS_PLUGIN_ZIP_CHECKSUM: Lazy<String> = Lazy::new(|| crate::utils::get_sha256_checksum(&PROCESS_PLUGIN_ZIP_BYTES));
 
-pub fn run_test_cli(args: Vec<&str>, environment: &TestEnvironment) -> Result<()> {
+#[derive(Debug, Error)]
+#[error("{inner:#}")]
+pub struct TestAppError {
+  asserted_exit_code: RefCell<bool>,
+  inner: AppError,
+}
+
+impl TestAppError {
+  #[track_caller]
+  pub fn assert_exit_code(&self, exit_code: i32) {
+    self.asserted_exit_code.replace(true);
+    assert_eq!(self.inner.exit_code, exit_code);
+  }
+}
+
+impl From<AppError> for TestAppError {
+  fn from(inner: AppError) -> Self {
+    Self {
+      asserted_exit_code: Default::default(),
+      inner,
+    }
+  }
+}
+
+impl From<anyhow::Error> for TestAppError {
+  fn from(inner: anyhow::Error) -> Self {
+    Self {
+      asserted_exit_code: Default::default(),
+      inner: inner.into(),
+    }
+  }
+}
+
+impl Drop for TestAppError {
+  fn drop(&mut self) {
+    if std::thread::panicking() || self.inner.exit_code <= 1 {
+      return;
+    }
+    if !self.asserted_exit_code.borrow().clone() {
+      panic!("Exit code must be asserted. Was: {}", self.inner.exit_code);
+    }
+  }
+}
+
+pub fn run_test_cli(args: Vec<&str>, environment: &TestEnvironment) -> Result<(), TestAppError> {
   run_test_cli_with_stdin(args, environment, TestStdInReader::default())
 }
 
-pub fn run_test_cli_with_stdin(args: Vec<&str>, environment: &TestEnvironment, stdin_reader: TestStdInReader) -> Result<()> {
+pub fn run_test_cli_with_stdin(args: Vec<&str>, environment: &TestEnvironment, stdin_reader: TestStdInReader) -> Result<(), TestAppError> {
   let mut args: Vec<String> = args.into_iter().map(String::from).collect();
   args.insert(0, String::from(""));
   environment.set_wasm_compile_result(COMPILATION_RESULT.clone());
-  let plugin_cache = Arc::new(PluginCache::new(environment.clone()));
-  let plugin_pools = Arc::new(PluginsCollection::new(environment.clone()));
-  let plugin_resolver = PluginResolver::new(environment.clone(), plugin_cache, plugin_pools.clone());
-  let args = parse_args(args, stdin_reader)?;
+  let plugin_cache = PluginCache::new(environment.clone());
+  let plugin_resolver = Rc::new(PluginResolver::new(environment.clone(), plugin_cache));
+  let args = parse_args(args, stdin_reader).map_err(|err| Into::<AppError>::into(err))?;
   environment.set_stdout_machine_readable(args.is_stdout_machine_readable());
-  environment.set_verbose(args.verbose);
+  environment.set_log_level(args.log_level);
 
   environment.run_in_runtime({
     let environment = environment.clone();
     async move {
-      let result = run_cli(&args, &environment, &plugin_resolver, plugin_pools.clone()).await;
-      plugin_pools.drop_and_shutdown_initialized().await;
-      result
+      let result = run_cli(&args, &environment, &plugin_resolver).await;
+      plugin_resolver.clear_and_shutdown_initialized().await;
+      Ok(result?)
     }
   })
 }
 
-pub fn get_test_process_plugin_zip_checksum() -> String {
-  crate::utils::get_sha256_checksum(&PROCESS_PLUGIN_ZIP_BYTES)
-}
-
 pub fn get_test_wasm_plugin_checksum() -> String {
   crate::utils::get_sha256_checksum(WASM_PLUGIN_BYTES)
 }
 
-pub fn get_test_process_plugin_checksum() -> String {
-  let zip_checksum = get_test_process_plugin_zip_checksum();
-  let ps_file_bytes = get_test_process_plugin_file_text(&zip_checksum).into_bytes();
-  crate::utils::get_sha256_checksum(&ps_file_bytes)
+pub struct TestProcessPluginFile(String);
+
+impl Default for TestProcessPluginFile {
+  fn default() -> Self {
+    TestProcessPluginFileBuilder::default().build()
+  }
 }
 
-pub fn get_test_process_plugin_file_text(zip_checksum: &str) -> String {
-  format!(
-    r#"{{
-"schemaVersion": 2,
-"name": "test-process-plugin",
-"version": "0.1.0",
-"windows-x86_64": {{
-    "reference": "https://github.com/dprint/test-process-plugin/releases/0.1.0/test-process-plugin.zip",
-    "checksum": "{0}"
-}},
-"linux-aarch64": {{
-    "reference": "https://github.com/dprint/test-process-plugin/releases/0.1.0/test-process-plugin.zip",
-    "checksum": "{0}"
-}},
-"linux-x86_64": {{
-    "reference": "https://github.com/dprint/test-process-plugin/releases/0.1.0/test-process-plugin.zip",
-    "checksum": "{0}"
-}},
-"darwin-x86_64": {{
-    "reference": "https://github.com/dprint/test-process-plugin/releases/0.1.0/test-process-plugin.zip",
-    "checksum": "{0}"
-}},
-"darwin-aarch64": {{
-    "reference": "https://github.com/dprint/test-process-plugin/releases/0.1.0/test-process-plugin.zip",
-    "checksum": "{0}"
-}}
-}}"#,
-    zip_checksum
-  )
+impl TestProcessPluginFile {
+  pub fn checksum(&self) -> String {
+    crate::utils::get_sha256_checksum(self.0.as_bytes())
+  }
+
+  pub fn text(&self) -> &str {
+    self.0.as_ref()
+  }
+}
+
+#[derive(Default)]
+pub struct TestProcessPluginFileBuilder {
+  schema_version: Option<u32>,
+  name: Option<String>,
+  version: Option<String>,
+  zip_checksum: Option<String>,
+}
+
+impl TestProcessPluginFileBuilder {
+  #[allow(unused)]
+  pub fn schema_version(mut self, schema_version: u32) -> Self {
+    self.schema_version = Some(schema_version);
+    self
+  }
+
+  #[allow(unused)]
+  pub fn name(mut self, name: &str) -> Self {
+    self.name = Some(name.to_string());
+    self
+  }
+
+  pub fn version(mut self, version: &str) -> Self {
+    self.version = Some(version.to_string());
+    self
+  }
+
+  pub fn zip_checksum(mut self, zip_checksum: &str) -> Self {
+    self.zip_checksum = Some(zip_checksum.to_string());
+    self
+  }
+
+  pub fn build(self) -> TestProcessPluginFile {
+    TestProcessPluginFile(format!(
+      r#"{{
+  "schemaVersion": {0},
+  "name": "{1}",
+  "version": "{2}",
+  "windows-x86_64": {{
+      "reference": "https://github.com/dprint/test-process-plugin/releases/0.1.0/test-process-plugin.zip",
+      "checksum": "{3}"
+  }},
+  "linux-aarch64": {{
+      "reference": "https://github.com/dprint/test-process-plugin/releases/0.1.0/test-process-plugin.zip",
+      "checksum": "{3}"
+  }},
+  "linux-x86_64": {{
+      "reference": "https://github.com/dprint/test-process-plugin/releases/0.1.0/test-process-plugin.zip",
+      "checksum": "{3}"
+  }},
+  "darwin-x86_64": {{
+      "reference": "https://github.com/dprint/test-process-plugin/releases/0.1.0/test-process-plugin.zip",
+      "checksum": "{3}"
+  }},
+  "darwin-aarch64": {{
+      "reference": "https://github.com/dprint/test-process-plugin/releases/0.1.0/test-process-plugin.zip",
+      "checksum": "{3}"
+  }}
+  }}"#,
+      self.schema_version.unwrap_or(2),
+      self.name.unwrap_or("test-process-plugin".to_string()),
+      self.version.unwrap_or("0.1.0".to_string()),
+      self.zip_checksum.unwrap_or(PROCESS_PLUGIN_ZIP_CHECKSUM.to_string())
+    ))
+  }
 }
 
 pub fn get_singular_formatted_text() -> String {
@@ -162,6 +262,7 @@ SUBCOMMANDS:
   output-format-times     Prints the amount of time it takes to format each file. Use this for debugging.
   clear-cache             Deletes the plugin cache directory.
   upgrade                 Upgrades the dprint executable.
+  completions             Generate shell completions script for dprint
   license                 Outputs the software license.
 
 More details at `dprint help <SUBCOMMAND>`
@@ -169,7 +270,7 @@ More details at `dprint help <SUBCOMMAND>`
 OPTIONS:
   -c, --config <config>          Path or url to JSON configuration file. Defaults to dprint.json(c) or .dprint.json(c) in current or ancestor directory when not provided.
       --plugins <urls/files>...  List of urls or file paths of plugins to use. This overrides what is specified in the config file.
-      --verbose                  Prints additional diagnostic information.
+  -L, --log-level <log-level>    Set log level [default: info] [possible values: debug, info, warn, error, silent]
 
 ENVIRONMENT VARIABLES:
   DPRINT_CACHE_DIR    Directory to store the dprint cache. Note that this
