@@ -6,23 +6,30 @@ use crossterm::event::KeyEvent;
 use crossterm::event::KeyModifiers;
 use crossterm::terminal;
 
-pub fn get_terminal_width() -> Option<u16> {
-  get_terminal_size().map(|(cols, _)| cols)
+#[derive(Debug, Copy, Clone)]
+pub struct TerminalSize {
+  pub cols: u16,
+  pub rows: u16,
 }
 
-/// Gets the terminal size (width/cols, height/rows).
-pub fn get_terminal_size() -> Option<(u16, u16)> {
+/// Gets the terminal size.
+pub fn get_terminal_size() -> Option<TerminalSize> {
   match crossterm::terminal::size() {
-    Ok(size) => Some(size),
+    Ok(size) => Some(TerminalSize { cols: size.0, rows: size.1 }),
     Err(_) => None,
   }
 }
 
 pub(crate) fn read_terminal_event() -> Result<Event> {
   // https://github.com/crossterm-rs/crossterm/issues/521
-  terminal::enable_raw_mode()?;
+  let was_raw_mode_enabled = terminal::is_raw_mode_enabled()?;
+  if !was_raw_mode_enabled {
+    terminal::enable_raw_mode()?;
+  }
   let result = read();
-  terminal::disable_raw_mode()?;
+  if !was_raw_mode_enabled {
+    terminal::disable_raw_mode()?;
+  }
   match result {
     Ok(Event::Key(KeyEvent {
       code: KeyCode::Char('c'),
